import logging
import os

from zygoat.components import SettingsComponent

from .secret_key import secret_key
from .database_config import database_config
from .debug_config import debug_config
from .installed_apps import installed_apps
from .allowed_hosts import allowed_hosts
from .cookies import cookies
from .drf_camelize import drf_camelize
from .reverse_proxy import reverse_proxy
from .env import env

log = logging.getLogger()

zygoat_settings_comment = """\"\"\"
This settings file is generated and updated by Zygoat and should not be edited
manually. Instead, update settings via this package's __init__.py.
\"\"\""""


class Settings(SettingsComponent):
    def create(self):
        log.info("Making python package for Django settings")
        os.mkdir(self.settings_directory)

        log.info("Moving the django settings into the settings package")
        os.rename(self.initial_settings_file_path, self.settings_file_path)

        log.info("Creating import for zygoat settings in __init__.py")
        with open(os.path.join(self.settings_directory, "__init__.py"), "a") as f:
            f.write(f"from .{SettingsComponent.MODULE_NAME} import *  # noqa\n")

        red = self.parse()

        log.info("Adding comment to Zygoat settings file")
        red[0].value = zygoat_settings_comment

        log.info("Dumping django settings file")
        self.dump(red)

    @property
    def installed(self):
        return os.path.exists(self.settings_file_path)


settings_sub_components = [
    secret_key,
    database_config,
    debug_config,
    installed_apps,
    allowed_hosts,
    cookies,
    drf_camelize,
    reverse_proxy,
    env,
]

settings = Settings(sub_components=settings_sub_components)



import logging

from zygoat.components import SettingsComponent

log = logging.getLogger()


class SecretKey(SettingsComponent):
    def create(self):
        red = self.parse()
        key_node = red.find("name", value="SECRET_KEY").parent

        log.info("Retrieving default secret key")
        default_key = key_node.value.to_python()

        log.info("Relocating default secret key")
        key_node.value = f"prod_required_env('DJANGO_SECRET_KEY', default='{default_key}')"

        log.info("Dumping secret key node")
        self.dump(red)

    @property
    def installed(self):
        red = self.parse()
        return (
            "prod_required_env(" in red.find("name", value="SECRET_KEY").parent.value.dumps()
        )


secret_key = SecretKey()



import logging
import os
import requests

from zygoat.components import Component

log = logging.getLogger()

file_name = ".gitignore"


class GitIgnore(Component):
    def create(self):
        with open(file_name, "a") as f:
            log.info("Retrieving node gitignore info")
            res = requests.get("https://gitignore.io/api/node")
            res.raise_for_status()

            f.write(res.text + "\n")
            f.write(".tern-port")

    @property
    def installed(self):
        if not os.path.exists(file_name):
            return False

        with open(file_name) as f:
            return "api/node" in f.read()


gitignore = GitIgnore()



import logging

from zygoat.components import SettingsComponent

log = logging.getLogger()

# If this string starts with a blank line, RedBaron completely ignores it
settings_string = """REST_FRAMEWORK = {
    "DEFAULT_RENDERER_CLASSES": (
        "djangorestframework_camel_case.render.CamelCaseJSONRenderer",
        "djangorestframework_camel_case.render.CamelCaseBrowsableAPIRenderer",
    ),
    "DEFAULT_PARSER_CLASSES": (
        "djangorestframework_camel_case.parser.CamelCaseFormParser",
        "djangorestframework_camel_case.parser.CamelCaseMultiPartParser",
        "djangorestframework_camel_case.parser.CamelCaseJSONParser",
    ),
}"""


class DRF_Camelize(SettingsComponent):
    def create(self):
        red = self.parse()

        red.extend(["\n", settings_string])

        log.info("Dumping DRF camelize configuration")
        self.dump(red)

    @property
    def installed(self):
        red = self.parse()
        node = red.find("name", value="REST_FRAMEWORK")

        if node is None:
            return False

        return "Camel" in red.find("name", value="REST_FRAMEWORK").parent.value.dumps()


drf_camelize = DRF_Camelize()



import logging

from zygoat.components import SettingsComponent

log = logging.getLogger()
log.setLevel(logging.INFO)


class DebugConfig(SettingsComponent):
    def create(self):
        red = self.parse()
        secret_key_index = red.index(red.find("name", value="SECRET_KEY").parent)

        log.info("Inserting PRODUCTION environment variable")
        red.insert(
            secret_key_index - 2, "PRODUCTION = env.bool('DJANGO_PRODUCTION', default=False)"
        )

        debug_node = red.find("name", value="DEBUG").parent

        log.info("Disabling debug in production")
        debug_node.value = "not PRODUCTION"

        log.info("Dumping debug configuration node")
        self.dump(red)

    @property
    def installed(self):
        red = self.parse()
        return "PRODUCTION" in red.find("name", value="DEBUG").parent.dumps()


debug_config = DebugConfig()



import importlib
import logging

from zygoat.constants import Phases, Projects
from zygoat.components import Component
from zygoat.config import yaml
from . import resources

log = logging.getLogger()
file_name = "docker-compose.yml"


class DockerCompose(Component):
    def _dump_config(self, data):
        with open(file_name, "w") as root_config:
            yaml.dump(data, root_config)

    def _load_config(self):
        with open(file_name) as root_config:
            return yaml.load(root_config.read())

    def create(self):
        log.info(f"Reading {file_name} from the repo")

        config = self._load_config()
        config["services"].update(
            yaml.load(importlib.resources.read_text(resources, file_name))
        )

        log.info("Dumping updated docker-compose config")
        self._dump_config(config)

    def update(self):
        self.call_phase(Phases.CREATE, force_create=True)

    def delete(self):
        config = self._load_config()

        log.info("Removing frontend service from config")
        del config["services"][Projects.FRONTEND]

        log.info("Dumping updated docker-compose config")
        self._dump_config(config)

    @property
    def installed(self):
        services = self._load_config()["services"]

        return Projects.FRONTEND in services


docker_compose = DockerCompose()
